@@ -1,7 +1,7 @@
 use crate::data::error_info::ErrorInfo;
 use crate::data::literal::ContentType;
 use crate::data::primitive::{closure::capture_variables, PrimitiveArray, PrimitiveObject};
-use crate::data::Position;
+use crate::data::{Position, warnings::DisplayWarnings};
 use crate::data::{ast::*, ArgsType, Data, Literal, MessageData, MSG};
 use crate::error_format::*;
 use crate::interpreter::{
@@ -21,17 +21,17 @@ use std::{collections::HashMap, sync::mpsc};
 
 fn exec_path_literal(
     literal: &mut Literal,
-    condition: bool,
+    dis_warnings: &DisplayWarnings,
     path: Option<&[(Interval, PathState)]>,
     data: &mut Data,
     msg_data: &mut MessageData,
     sender: &Option<mpsc::Sender<MSG>>,
 ) -> Result<Literal, ErrorInfo> {
     if let Some(path) = path {
-        let path = resolve_path(path, condition, data, msg_data, sender)?;
+        let path = resolve_path(path, dis_warnings, data, msg_data, sender)?;
         let (mut new_literal, ..) = exec_path_actions(
             literal,
-            condition,
+            dis_warnings,
             None,
             &Some(path),
             &ContentType::get(&literal),
@@ -57,7 +57,7 @@ fn exec_path_literal(
 
 pub fn expr_to_literal(
     expr: &Expr,
-    condition: bool,
+    dis_warnings: &DisplayWarnings,
     path: Option<&[(Interval, PathState)]>,
     data: &mut Data,
     msg_data: &mut MessageData,
@@ -65,12 +65,12 @@ pub fn expr_to_literal(
 ) -> Result<Literal, ErrorInfo> {
     match expr {
         Expr::ObjectExpr(ObjectType::As(name, var)) => {
-            let value = expr_to_literal(var, condition, None, data, msg_data, sender)?;
+            let value = expr_to_literal(var, dis_warnings, None, data, msg_data, sender)?;
             data.step_vars.insert(name.ident.to_owned(), value.clone());
             Ok(value)
         }
         Expr::PathExpr { literal, path } => {
-            expr_to_literal(literal, condition, Some(path), data, msg_data, sender)
+            expr_to_literal(literal, dis_warnings, Some(path), data, msg_data, sender)
         }
         Expr::ObjectExpr(ObjectType::BuiltIn(Function {
             name,
@@ -79,7 +79,7 @@ pub fn expr_to_literal(
         })) => {
             let mut literal = resolve_object(&name, args, *interval, data, msg_data, sender)?;
 
-            exec_path_literal(&mut literal, condition, path, data, msg_data, sender)
+            exec_path_literal(&mut literal, dis_warnings, path, data, msg_data, sender)
         }
         Expr::MapExpr {
             object,
@@ -91,11 +91,11 @@ pub fn expr_to_literal(
             for (key, value) in object.iter() {
                 map.insert(
                     key.to_owned(),
-                    expr_to_literal(&value, condition, None, data, msg_data, sender)?,
+                    expr_to_literal(&value, dis_warnings, None, data, msg_data, sender)?,
                 );
             }
             let mut literal = PrimitiveObject::get_literal(&map, range_interval.to_owned());
-            exec_path_literal(&mut literal, condition, path, data, msg_data, sender)
+            exec_path_literal(&mut literal, dis_warnings, path, data, msg_data, sender)
         }
         Expr::ComplexLiteral(vec, range_interval) => {
             let mut string = get_string_from_complex_string(
@@ -105,30 +105,30 @@ pub fn expr_to_literal(
                 msg_data,
                 sender,
             )?;
-            exec_path_literal(&mut string, condition, path, data, msg_data, sender)
+            exec_path_literal(&mut string, dis_warnings, path, data, msg_data, sender)
         }
         Expr::VecExpr(vec, range_interval) => {
             let mut array = vec![];
             for value in vec.iter() {
                 array.push(expr_to_literal(
-                    value, condition, None, data, msg_data, sender,
+                    value, dis_warnings, None, data, msg_data, sender,
                 )?)
             }
             let mut literal = PrimitiveArray::get_literal(&array, range_interval.to_owned());
-            exec_path_literal(&mut literal, condition, path, data, msg_data, sender)
+            exec_path_literal(&mut literal, dis_warnings, path, data, msg_data, sender)
         }
         Expr::PostfixExpr(postfix, expr) => {
             let mut literal = evaluate_postfix(postfix, expr, data, msg_data, sender)?;
-            exec_path_literal(&mut literal, condition, path, data, msg_data, sender)
+            exec_path_literal(&mut literal, dis_warnings, path, data, msg_data, sender)
         }
         Expr::InfixExpr(infix, exp_1, exp_2) => {
             let mut literal = evaluate_condition(infix, exp_1, exp_2, data, msg_data, sender)?;
-            exec_path_literal(&mut literal, condition, path, data, msg_data, sender)
+            exec_path_literal(&mut literal, dis_warnings, path, data, msg_data, sender)
         }
         Expr::LitExpr { literal, .. } => {
             let mut new_value = exec_path_literal(
                 &mut literal.clone(),
-                condition,
+                dis_warnings,
                 path,
                 data,
                 msg_data,
@@ -141,7 +141,7 @@ pub fn expr_to_literal(
         }
         Expr::IdentExpr(var, ..) => Ok(get_var(
             var.to_owned(),
-            condition,
+            dis_warnings,
             path,
             data,
             msg_data,
@@ -158,6 +158,7 @@ pub fn resolve_fn_args(
     expr: &Expr,
     data: &mut Data,
     msg_data: &mut MessageData,
+    dis_warnings: &DisplayWarnings,
     sender: &Option<mpsc::Sender<MSG>>,
 ) -> Result<ArgsType, ErrorInfo> {
     match expr {
@@ -180,7 +181,7 @@ pub fn resolve_fn_args(
                         };
                         named_args = true;
 
-                        let literal = expr_to_literal(var, false, None, data, msg_data, sender)?;
+                        let literal = expr_to_literal(var, dis_warnings, None, data, msg_data, sender)?;
                         map.insert(name.ident.to_owned(), literal);
                     }
                     expr => {
@@ -191,7 +192,7 @@ pub fn resolve_fn_args(
                                 ERROR_EXPR_TO_LITERAL.to_owned(), // TODO: error mix of named args and anonymous args
                             ));
                         }
-                        let literal = expr_to_literal(expr, false, None, data, msg_data, sender)?;
+                        let literal = expr_to_literal(expr, dis_warnings, None, data, msg_data, sender)?;
                         map.insert(format!("arg{}", index), literal);
                     }
                 }
