@@ -3,7 +3,7 @@ use crate::data::{
     ast::{Expr, Postfix, Infix},
     Literal, Data, MessageData, MSG,
     primitive::boolean::PrimitiveBoolean,
-    position::Position,
+    position::Position, warnings::DisplayWarnings,
 };
 use crate::interpreter::{
     variable_handler::{
@@ -55,6 +55,7 @@ pub fn evaluate_infix(
                 Ok(primitive) => Ok(Literal {
                     content_type: primitive.get_type().to_string(),
                     primitive,
+                    additional_info: None,
                     interval: lhs.interval,
                 }),
                 Err(err) => Err(gen_error_info(Position::new(lhs.interval, flow_name), err)),
@@ -67,6 +68,7 @@ pub fn evaluate_infix(
                 Ok(primitive) => Ok(Literal {
                     content_type: primitive.get_type().to_string(),
                     primitive,
+                    additional_info: None,
                     interval: lhs.interval,
                 }),
                 Err(err) => Err(gen_error_info(Position::new(lhs.interval, flow_name), err)),
@@ -79,6 +81,7 @@ pub fn evaluate_infix(
                 Ok(primitive) => Ok(Literal {
                     content_type: primitive.get_type().to_string(),
                     primitive,
+                    additional_info: None,
                     interval: lhs.interval,
                 }),
                 Err(err) => Err(gen_error_info(Position::new(lhs.interval, flow_name), err)),
@@ -92,6 +95,7 @@ pub fn evaluate_infix(
                 Ok(primitive) => Ok(Literal {
                     content_type: primitive.get_type().to_string(),
                     primitive,
+                    additional_info: None,
                     interval: lhs.interval,
                 }),
                 Err(err) => Err(gen_error_info(Position::new(lhs.interval, flow_name), err)),
@@ -104,6 +108,7 @@ pub fn evaluate_infix(
                 Ok(primitive) => Ok(Literal {
                     content_type: primitive.get_type().to_string(),
                     primitive,
+                    additional_info: None,
                     interval: lhs.interval,
                 }),
                 Err(err) => Err(gen_error_info(Position::new(lhs.interval, flow_name), err)),
@@ -137,7 +142,7 @@ pub fn evaluate_postfix(
     msg_data: &mut MessageData,
     sender: &Option<mpsc::Sender<MSG>>,
 )  -> Result<Literal, ErrorInfo> {
-    let value = valid_literal(expr_to_literal(expr, true, None, data, msg_data, sender));
+    let value = valid_literal(expr_to_literal(expr, &DisplayWarnings::Off, None, data, msg_data, sender));
     let interval = interval_from_expr(expr);
 
     if postfixes.len() % 2  == 0 {
