@@ -1,4 +1,4 @@
-use crate::data::{ast::*, tokens::*};
+use crate::data::{ast::*, tokens::*, csml_logs::LogLvl};
 use crate::error_format::{
     gen_nom_failure, ERROR_ACTION_ARGUMENT, ERROR_REMEMBER, ERROR_RETURN, ERROR_USE,
 };
@@ -18,11 +18,12 @@ use crate::parser::{
 
 use nom::{
     branch::alt,
-    bytes::complete::tag,
+    bytes::complete::{tag, take_while1},
     combinator::{opt},
-    error::ParseError,
-    multi::separated_list,
-    sequence::{preceded, terminated, tuple},
+    error::{ParseError, ContextError, ErrorKind},
+    multi::{separated_list0},
+    sequence::{preceded, terminated, tuple, },
+
     Err, IResult,
 };
 
@@ -32,7 +33,7 @@ use nom::{
 
 fn addition_assignment<'a, E>(s: Span<'a>) -> IResult<Span<'a>, AssignType, E>
 where
-    E: ParseError<Span<'a>>,
+    E: ParseError<Span<'a>> + ContextError<Span<'a>>,
 {
     let (rest, ..) = tag(ADDITION_ASSIGNMENT)(s)?;
     Ok((rest, AssignType::AdditionAssignment))
@@ -40,7 +41,7 @@ where
 
 fn subtraction_assignment<'a, E>(s: Span<'a>) -> IResult<Span<'a>, AssignType, E>
 where
-    E: ParseError<Span<'a>>,
+    E: ParseError<Span<'a>> + ContextError<Span<'a>>,
 {
     let (rest, ..) = tag(SUBTRACTION_ASSIGNMENT)(s)?;
     Ok((rest, AssignType::SubtractionAssignment))
@@ -48,7 +49,7 @@ where
 
 fn assignment<'a, E>(s: Span<'a>) -> IResult<Span<'a>, AssignType, E>
 where
-    E: ParseError<Span<'a>>,
+    E: ParseError<Span<'a>> + ContextError<Span<'a>>,
 {
     let (rest, ..) = tag(ASSIGN)(s)?;
     Ok((rest, AssignType::Assignment))
@@ -56,7 +57,7 @@ where
 
 fn parse_assignation<'a, E>(s: Span<'a>) -> IResult<Span<'a>, (Identifier, Box<Expr>), E>
 where
-    E: ParseError<Span<'a>>,
+    E: ParseError<Span<'a>> + ContextError<Span<'a>>,
 {
     let (s, name) = parse_idents_assignation(s)?;
     let (s, _) = preceded(comment, tag(ASSIGN))(s)?;
@@ -67,7 +68,7 @@ where
 
 fn parse_assignation_with_path<'a, E>(s: Span<'a>) -> IResult<Span<'a>, Expr, E>
 where
-    E: ParseError<Span<'a>>,
+    E: ParseError<Span<'a>> + ContextError<Span<'a>>,
 {
     let (s, name) = parse_idents_assignation(s)?;
     let (s, ident) = parse_path(s, Expr::IdentExpr(name))?;
@@ -83,7 +84,7 @@ where
 
 fn parse_remember_as<'a, E>(s: Span<'a>) -> IResult<Span<'a>, (Identifier, Box<Expr>), E>
 where
-    E: ParseError<Span<'a>>,
+    E: ParseError<Span<'a>> + ContextError<Span<'a>>,
 {
     let (s, operator) = parse_operator(s)?;
 
@@ -95,7 +96,7 @@ where
 
 fn parse_forget_all<'a, E>(s: Span<'a>) -> IResult<Span<'a>, ForgetMemory, E>
 where
-    E: ParseError<Span<'a>>,
+    E: ParseError<Span<'a>> + ContextError<Span<'a>>,
 {
     let (s, _) = tag("*")(s)?;
 
@@ -104,7 +105,7 @@ where
 
 fn parse_forget_single<'a, E>(s: Span<'a>) -> IResult<Span<'a>, ForgetMemory, E>
 where
-    E: ParseError<Span<'a>>,
+    E: ParseError<Span<'a>> + ContextError<Span<'a>>,
 {
     let (s, ident) = parse_idents_usage(s)?;
 
@@ -113,13 +114,13 @@ where
 
 fn parse_forget_list<'a, E>(s: Span<'a>) -> IResult<Span<'a>, ForgetMemory, E>
 where
-    E: ParseError<Span<'a>>,
+    E: ParseError<Span<'a>> + ContextError<Span<'a>>,
 {
     let (s, (vec, _)) = preceded(
         tag(L_BRACKET),
         terminated(
             tuple((
-                separated_list(preceded(comment, tag(COMMA)), parse_idents_usage), 
+                separated_list0(preceded(comment, tag(COMMA)), parse_idents_usage), 
                 opt(preceded(comment, tag(COMMA))),
             )),
             preceded(comment, parse_r_bracket),
@@ -131,8 +132,8 @@ where
 
 fn parse_action_argument<'a, E, F, G>(s: Span<'a>, func: F) -> IResult<Span<'a>, G, E>
 where
-    E: ParseError<Span<'a>>,
-    F: Fn(Span<'a>) -> IResult<Span<'a>, G, E>,
+    E: ParseError<Span<'a>> + ContextError<Span<'a>>,
+    F: FnMut(Span<'a>) -> IResult<Span<'a>, G, E>,
 {
     match preceded(comment, func)(s) {
         Ok(value) => Ok(value),
@@ -150,7 +151,7 @@ where
 
 fn parse_do<'a, E>(s: Span<'a>) -> IResult<Span<'a>, Expr, E>
 where
-    E: ParseError<Span<'a>>,
+    E: ParseError<Span<'a>> + ContextError<Span<'a>>,
 {
     let (s, name) = preceded(comment, get_string)(s)?;
     let (s, ..) = get_tag(name, DO)(s)?;
@@ -173,7 +174,7 @@ where
 
 fn parse_remember<'a, E>(s: Span<'a>) -> IResult<Span<'a>, Expr, E>
 where
-    E: ParseError<Span<'a>>,
+    E: ParseError<Span<'a>> + ContextError<Span<'a>>,
 {
     let (s, name) = preceded(comment, get_string)(s)?;
     let (s, ..) = get_tag(name, REMEMBER)(s)?;
@@ -189,7 +190,7 @@ where
 
 fn parse_forget<'a, E>(s: Span<'a>) -> IResult<Span<'a>, Expr, E>
 where
-    E: ParseError<Span<'a>>,
+    E: ParseError<Span<'a>> + ContextError<Span<'a>>,
 {
     let (s, name) = preceded(comment, get_string)(s)?;
     let (s, interval) = get_interval(s)?;
@@ -214,7 +215,7 @@ where
 
 fn parse_say<'a, E>(s: Span<'a>) -> IResult<Span<'a>, Expr, E>
 where
-    E: ParseError<Span<'a>>,
+    E: ParseError<Span<'a>> + ContextError<Span<'a>>,
 {
     let (s, name) = preceded(comment, get_string)(s)?;
     let (s, ..) = get_tag(name, SAY)(s)?;
@@ -229,7 +230,7 @@ where
 
 fn parse_debug<'a, E>(s: Span<'a>) -> IResult<Span<'a>, Expr, E>
 where
-    E: ParseError<Span<'a>>,
+    E: ParseError<Span<'a>> + ContextError<Span<'a>>,
 {
     let (s, name) = preceded(comment, get_string)(s)?;
     let (s, mut interval) = get_interval(s)?;
@@ -248,10 +249,56 @@ where
     ))
 }
 
+
+pub fn parse_log_lvl<'a, E>(s: Span<'a>) -> IResult<Span<'a>, LogLvl, E>
+where
+    E: ParseError<Span<'a>> + ContextError<Span<'a>>,
+{
+    let (s, _) = comment(s)?;
+
+    let (rest, string) = take_while1(|c: char| c == '_' || c == '\\' || c.is_alphanumeric())(s)?;
+
+    match string.fragment().to_lowercase().as_str() {
+        "error" => Ok((rest, LogLvl::Error)),
+        "warn" => Ok((rest, LogLvl::Warn)),
+        "info" => Ok((rest, LogLvl::Info)),
+        "debug" => Ok((rest, LogLvl::Debug)),
+        "trace" => Ok((rest, LogLvl::Trace)),
+        _      => Err(Err::Error(E::from_error_kind(s, ErrorKind::Tag)))
+    }
+}
+
+fn parse_log<'a, E>(s: Span<'a>) -> IResult<Span<'a>, Expr, E>
+where
+    E: ParseError<Span<'a>> + ContextError<Span<'a>>,
+{
+    let (s, name) = preceded(comment, get_string)(s)?;
+    let (s, mut interval) = get_interval(s)?;
+    let (s, ..) = get_tag(name, LOG_ACTION)(s)?;
+
+    let (s, log_lvl) =  match opt(parse_log_lvl)(s)? {
+        (s, Some(log_lvl)) => (s, log_lvl),
+        (s, None) => (s, LogLvl::Info)
+    };
+
+    let (s, expr) = parse_action_argument(s, parse_operator)?;
+    let (s, end) = get_interval(s)?;
+
+    interval.add_end(end);
+
+    // this vec is temporary until a solution for multiple arguments in debug is found
+    let vec = Expr::VecExpr(vec![expr], interval);
+
+    Ok((
+        s,
+        Expr::ObjectExpr(ObjectType::Log{expr: Box::new(vec), interval, log_lvl}),
+    ))
+}
+
 //TODO: deprecate use
 fn parse_use<'a, E>(s: Span<'a>) -> IResult<Span<'a>, Expr, E>
 where
-    E: ParseError<Span<'a>>,
+    E: ParseError<Span<'a>> + ContextError<Span<'a>>,
 {
     let (s, name) = preceded(comment, get_string)(s)?;
     let (s, ..) = get_tag(name, USE)(s)?;
@@ -271,7 +318,7 @@ where
 
 fn parse_hold<'a, E>(s: Span<'a>) -> IResult<Span<'a>, Expr, E>
 where
-    E: ParseError<Span<'a>>,
+    E: ParseError<Span<'a>> + ContextError<Span<'a>>,
 {
     let (s, inter) = preceded(comment, get_interval)(s)?;
     let (s, name) = get_string(s)?;
@@ -286,7 +333,7 @@ where
 
 fn parse_break<'a, E>(s: Span<'a>) -> IResult<Span<'a>, Expr, E>
 where
-    E: ParseError<Span<'a>>,
+    E: ParseError<Span<'a>> + ContextError<Span<'a>>,
 {
     let (s, inter) = preceded(comment, get_interval)(s)?;
     let (s, name) = get_string(s)?;
@@ -301,7 +348,7 @@ where
 
 fn parse_continue<'a, E>(s: Span<'a>) -> IResult<Span<'a>, Expr, E>
 where
-    E: ParseError<Span<'a>>,
+    E: ParseError<Span<'a>> + ContextError<Span<'a>>,
 {
     let (s, inter) = preceded(comment, get_interval)(s)?;
     let (s, name) = get_string(s)?;
@@ -316,7 +363,7 @@ where
 
 fn parse_return<'a, E>(s: Span<'a>) -> IResult<Span<'a>, Expr, E>
 where
-    E: ParseError<Span<'a>>,
+    E: ParseError<Span<'a>> + ContextError<Span<'a>>,
 {
     let (s, name) = preceded(comment, get_string)(s)?;
     let (s, ..) = get_tag(name, RETURN)(s)?;
@@ -340,12 +387,13 @@ where
 
 pub fn parse_root_functions<'a, E>(s: Span<'a>) -> IResult<Span<'a>, Expr, E>
 where
-    E: ParseError<Span<'a>>,
+    E: ParseError<Span<'a>> + ContextError<Span<'a>>,
 {
     alt((
         // common actions
         parse_do,
         parse_debug,
+        parse_log,
         parse_if,
         parse_foreach,
         parse_while,
